//
//  CVUValue.swift
//  MemriDatabase
//
//  Created by T Brennan on 7/12/20.
//

import Foundation

/// A value of a CVU property that can be serialised both to CVU format and to JSON
enum CVUValue: Equatable, AutoCodable {
    case expression(ExpressionNode)
    case constant(Constant)
    case item(StringUUID)
    case array([CVUValue])
    case dictionary([String: CVUValue])
    case subdefinition(CVUDefinitionContent)
    
    func getSubdefinition() -> CVUDefinitionContent? {
        switch self {
        case .subdefinition(let x):
            return x
        default:
            return nil
        }
    }
}

extension CVUValue: CVUStringConvertible {
    func toCVUString(depth: Int, tab: String, includeInitialTab: Bool) -> String {
        switch self {
        case .expression(let expression):
            let innerString = expression.toCVUString()
            switch expression {
            case .stringMode:
                return "\"\(innerString)\""
            default:
                return "{{\(innerString)}}"
            }
        case .constant(let constant):
            return constant.toCVUString()
        case .item(let uid):
            return "{{item(\(uid.uid))}}"
        case .array(let values):
            return values.toCVUString(depth: depth, tab: tab, includeInitialTab: false)
        case .dictionary(let dict):
            return "{\n\(dict.toCVUString(depth: depth, tab: tab, includeInitialTab: true))\n}"
        case .subdefinition(let x):
            return x.toCVUString(depth: depth, tab: tab, includeInitialTab: includeInitialTab)
        }
        
    }
}
