//
//  CVUValue_LookupNode.swift
//  Memri
//
//  Created by T Brennan on 28/1/21.
//

import Foundation

extension CVUValue {
    /// A single node in a CVU Expression lookup. The `default` node (`.` in CVU) represents the current item in the CVU context
    struct LookupNode: Equatable, AutoCodable {
        var name: String
        var isArray: Bool = false
        var type: LookupType
        
        static var `default` = LookupNode(name: "@@DEFAULT@@", type: .default)
        static var `localized` = LookupNode(name: "@@LOCALIZED@@", type: .`localized`)
    }
    
    enum LookupType: Equatable, AutoCodable {
        case `default`
        case `localized`
        case lookup(subExpression: ExpressionNode? = nil)
        case function(args: [ExpressionNode])
        
        typealias ExpressionNode = CVUValue.ExpressionNode
    }
}


extension CVUValue.LookupNode {
    func toCVUString() -> String {
        switch type {
        case .default:
            return ""
        case .function(let args):
            return "\(name)(\(args.map { $0.toCVUString() }.joined(separator: ", ")))"
        default:
            return "\(name)\(isArray ? "[]" : "")"
        }
    }
}
