//
//  StringUUID.swift
//  Memri
//
//  Created by T Brennan on 28/1/21.
//

import Foundation
import GRDB

/// This is the type used to represent UIDs within the iOS app. It can accept any string in its initialiser
/// Creating a new StringUUID using .init() will automatically generate a UUID-string that is guaranteed to be unique.
struct StringUUID: DatabaseValueConvertible, Hashable {
    var uid: String = UUID().uuidString
    
    var databaseValue: DatabaseValue {
        uid.databaseValue
    }
    
    static func fromDatabaseValue(_ dbValue: DatabaseValue) -> StringUUID? {
        String.fromDatabaseValue(dbValue).map(StringUUID.init(uid:))
    }
}

extension StringUUID: Codable {
    init(from decoder: Decoder) throws {
        uid = try decoder.singleValueContainer().decode(String.self)
    }
    
    func encode(to encoder: Encoder) throws {
        var container = encoder.singleValueContainer()
        try container.encode(uid)
    }
}
