//
//  PluginHandler.swift
//  Memri

import Foundation

class PluginHandler {
    public static func start(plugin: ItemRecord, runner: ItemRecord, sceneController: SceneController, context: CVUContext) {
        AppController.shared.pubsubController.startObservingItemProperty(runner, property: "state", desiredValue: nil) { (newValue, error) in
            guard case let .string(state) = newValue else {
                return
            }
            
            switch state {
                case "userActionNeeded":
                    presentCVUforPlugin(plugin: plugin, runner: runner, sceneController: sceneController, context: context)
            default:
                break
            }
        }
        
        try? AppController.shared.syncController.sync()
    }
    
    static func presentCVUforPlugin(plugin: ItemRecord, runner: ItemRecord, sceneController: SceneController, context: CVUContext) {
        guard let runnerRowId = runner.rowId,
              let view = plugin.edgeItem("view"),
              case let .string(cvuContent) = view.propertyValue("definition"),
              let cvuDefinition = try? CVUController.parseCVU(string: cvuContent).first?.parsed else {
            AppController.shared.pubsubController.stopObservingItemProperty(runner, property: "state")
            return
        }
                
        let item = ItemRecord(type: "Account")
        item.syncState = .skip // Don't sync it yet
        try? item.save()
        guard let itemRowId = item.rowId,
              let edge = try? EdgeRecord.createFor(source: runnerRowId, type: "account", target: itemRowId) else {
            return
        }
        
        edge.syncState = .skip // Don't sync it yet
        try? edge.save()

        sceneController.navigateToNewContext(defaultDefinition: cvuDefinition)
    }
}
