//
//  DateInterval.swift
//  Memri
//
//  Created by T Brennan on 31/1/21.
//

import Foundation
import SwiftUI

extension Date {
    static var oneWeekAgo: Date {
        Calendar.current.date(byAdding: .day, value: -7, to: Date()) ?? Date()
    }
    
    var timeDelta: String? {
        let formatter = DateComponentsFormatter()
        formatter.unitsStyle = .full
        formatter.maximumUnitCount = 1
        formatter.allowedUnits = [.year, .month, .day, .hour, .minute, .second]

        guard let deltaString = formatter.string(from: self, to: Date()) else {
            return nil
        }
        return deltaString
    }
    
    var timestampString: String? {
        guard let timeString = timeDelta else {
            return nil
        }
        let formatString = NSLocalizedString("%@ ago", comment: "")
        return String(format: formatString, timeString)
    }

    func formatted(_ dateFormat: String? = nil) -> String {
        let showTimeAgo = UserDefaults.standard.bool(forKey: "/user/general/gui/showDateAgo")
        guard showTimeAgo == false else {
            return self.timestampString ?? ""
        }
        
        let format = dateFormat ?? UserDefaults.standard.string(forKey: "/user/formatting/date") ?? "yyyy/MM/dd HH:mm"

        // Compare against 36 hours ago
        if self.timeIntervalSince(Date(timeIntervalSinceNow: -129_600)) < 0
        {
            let dateFormatter = DateFormatter()
            dateFormatter.dateFormat = format
            dateFormatter.locale = Locale(identifier: "en_US")
            dateFormatter.timeZone = TimeZone(secondsFromGMT: 0)
            return dateFormatter.string(from: self)
        }
        else {
            return self.timestampString ?? ""
        }
    }
}
