//
//  FileHelper.swift
//  MemriDatabase
//
//  Created by T Brennan on 20/12/20.
//

import Foundation
import CryptoKit

enum FileHelper {
    static func getSHAHash(url fileURL: URL) throws -> String {
        guard let stream = InputStream(url: fileURL) else {
            throw StringError(description: "Hashing error: could not open file at url: \(fileURL)")
        }
        var hasher = CryptoKit.SHA256()
        stream.open()
        let bufferSize = 512
        let buffer = UnsafeMutablePointer<UInt8>
                     .allocate(capacity: bufferSize)
        while stream.hasBytesAvailable {
            let read = stream.read(buffer, maxLength: bufferSize)
            let bufferPointer = UnsafeRawBufferPointer(start: buffer,
                                                       count: read)
            hasher.update(bufferPointer: bufferPointer)
        }
        stream.close()
        let digest = hasher.finalize()
        let stringHash = digest.map { String(format: "%02hhx", $0) }.joined()
        return stringHash
    }
}
