//
//  CVUElementView.swift
//  MemriDatabase
//
//  Created by T Brennan on 7/1/21.
//

import SwiftUI

/// This view is used to display CVU elements (and is used in a nested fashion to display their children)
struct CVUElementView: View {
    var nodeResolver: CVUUINodeResolver
    
    @ViewBuilder
    func resolvedComponent() -> some View {
        switch nodeResolver.node.type {
        case .ForEach:
            CVU_ForEach(nodeResolver: nodeResolver)
        case .HStack:
            CVU_HStack(nodeResolver: nodeResolver)
        case .VStack:
            CVU_VStack(nodeResolver: nodeResolver)
        case .ZStack:
            CVU_ZStack(nodeResolver: nodeResolver)
        case .Text:
            CVU_Text(nodeResolver: nodeResolver)
        case .Image:
            CVU_Image(nodeResolver: nodeResolver)
        case .Map:
            CVU_Map(nodeResolver: nodeResolver)
        case .SmartText:
            CVU_SmartText(nodeResolver: nodeResolver)
        case .Textfield:
            CVU_TextField(nodeResolver: nodeResolver)
        case .Toggle:
            CVU_Toggle(nodeResolver: nodeResolver)
        case .Button:
            CVU_Button(nodeResolver: nodeResolver)
        case .Divider:
            Divider()
        case .HorizontalLine:
            HorizontalLine()
        case .Circle:
            CVU_Shape.Circle(nodeResolver: nodeResolver)
        case .Rectangle:
            CVU_Shape.Rectangle(nodeResolver: nodeResolver)
        case .HTMLView:
            CVU_HTMLView(nodeResolver: nodeResolver)
        case .TimelineItem:
            CVU_TimelineItem(nodeResolver: nodeResolver)
        case .FileThumbnail:
            CVU_FileThumbnail(nodeResolver: nodeResolver)
        case .Spacer:
            Spacer()
        case .Empty:
            EmptyView()
        case .FlowStack:
            CVU_FlowStack(nodeResolver: nodeResolver)
        case .Grid:
            CVU_Grid(nodeResolver: nodeResolver)
        case .EditorRow:
            CVU_EditorRow(nodeResolver: nodeResolver)
        case .SubView:
            CVU_SubView(nodeResolver: nodeResolver)
        case .MemriButton:
            CVU_MemriButton(nodeResolver: nodeResolver)
        case .ActionButton:
            CVU_ActionButton(nodeResolver: nodeResolver)
        case .MessageComposer:
            CVU_MessageComposer(nodeResolver: nodeResolver)

//        case .Picker:
//            picker
//        case .EditorSection:
//            CVU_EditorSection(nodeResolver: nodeResolver)
//        case .MemriButton:
//            CVU_MemriButton(nodeResolver: nodeResolver)
        default:
            Text("\(nodeResolver.node.type.rawValue) not implemented yet.")
        }
    }
    
    var needsModifier: Bool {
        switch nodeResolver.node.type {
        case .Empty, .ForEach, .Spacer, .Divider, .FlowStack: return false
        default: return true
        }
    }
    
    @ViewBuilder
    public var body: some View {
        if nodeResolver.propertyResolver.showNode {
            resolvedComponent()
                .if(needsModifier) {
                    $0.modifier(CVU_AppearanceModifier(nodeResolver: nodeResolver))
                }
        }
    }
}

//struct CVUElementView_Previews: PreviewProvider {
//    static var previews: some View {
//        CVUElementView(type: .Text)
//    }
//}
