//
//  CVU_ActionButton.swift
//  Memri
//
//  Created by Chaitanya Pandit on 29/03/21.
//

import SwiftUI

/// A CVU element for displaying a button
/// - Use the `onPress` property to provide a CVU Action for the button
struct CVU_ActionButton: View {
    var nodeResolver: CVUUINodeResolver
    @EnvironmentObject var sceneController: SceneController

    var action: CVUAction? {
        guard let action = self.nodeResolver.propertyResolver.action(key: "onPress") else { return nil }

        return action
    }
    
    func onPress() {
        guard let actions = self.nodeResolver.propertyResolver.actions(key: "onPress") else { return }
        for action in actions {
            action.execute(sceneController: sceneController, context: nodeResolver.context, completion: nil)
        }
    }
    
    var title: String {
        guard let action = self.action as? CVUAction_OpenViewByName,
              let titleVal = action.vars["title"],
              case let .constant(.string(title)) = titleVal else {
            return ""
        }
        
        return title
    }
    
    var viewArguments: CVUViewArguments? {
        guard let action = self.action as? CVUAction_OpenViewByName,
              let argsValue = action.vars["viewArguments"],
              case let .subdefinition(args) = argsValue,
              let currentItem = nodeResolver.context.currentItem else {
            return nil
        }
        
        var properties = args.properties
        properties["subject"] = .item(currentItem.id)
            
        return CVUViewArguments(args: properties, argumentItem: nodeResolver.context.currentItem)
    }
    
    @State var isShowing = false

    var body: some View {
        if let validAction = self.action as? CVUAction_OpenViewByName {
            Button(action: {
                self.isShowing = true
            }) {
                Text(title)
                    .font(.subheadline)
                    .foregroundColor(.black)
            }
            .sheet(isPresented: $isShowing) {
                if let viewContext = validAction.getViewContext(context: CVUContext(currentItem:nodeResolver.context.currentItem, viewName: validAction.viewName, rendererName: validAction.renderer, viewArguments: viewArguments)) {
                    BrowserView(context: viewContext)
                } else {
                    Text("TODO: ActionPopupButton")
                }
            }
        } else if let starAction = self.action as? CVUAction_Star {
            Button(action: {
                withAnimation {
                    starAction.execute(sceneController: sceneController, context: nodeResolver.context, completion: nil)
                }
            }) {
                Image(systemName: (nodeResolver.context.currentItem?.propertyValue("starred")?.asBool() ?? false) == true ? "star.fill" : "star")
            }
        }  else {
            Text("TODO: CVU_ActionButton")
        }
    }
}
