//
// CVU_AppearanceModifier.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// A SwiftUI modifier used to apply common CVU properties (such as sizing, padding, colors, opacity, etc)
struct CVU_AppearanceModifier: ViewModifier {
    var nodeResolver: CVUUINodeResolver

    var shape: some InsettableShape {
        RoundedRectangle(cornerRadius: nodeResolver.propertyResolver.cornerRadius())
    }

    func body(content: Content) -> some View {
        content
            .frame(
                minWidth: nodeResolver.propertyResolver.minWidth,
                maxWidth: nodeResolver.propertyResolver.maxWidth,
                minHeight: nodeResolver.propertyResolver.minHeight,
                maxHeight: nodeResolver.propertyResolver.maxHeight
            )
            .foregroundColor(nodeResolver.propertyResolver.color()?.color)
            .font(nodeResolver.propertyResolver.font().font)
            .multilineTextAlignment(nodeResolver.propertyResolver.textAlignment())
            .lineLimit(nodeResolver.propertyResolver.lineLimit)
            .padding(nodeResolver.propertyResolver.padding)
            .if(nodeResolver.propertyResolver.cornerRadius() > 0) { $0.clipShape(shape) }
            .background(
                shape
                    .fill(nodeResolver.propertyResolver.backgroundColor?.color ?? .clear)
                    .ifLet(nodeResolver.propertyResolver.shadow) { $0.shadow(radius: $1) }
            )
            .overlay(
                shape
                    .strokeBorder(nodeResolver.propertyResolver.borderColor?.color ?? .clear)
            )
            .offset(nodeResolver.propertyResolver.offset)
            .opacity(nodeResolver.propertyResolver.opacity)
            .padding(nodeResolver.propertyResolver.margin)
            .ifLet(nodeResolver.propertyResolver.zIndex) { $0.zIndex($1) }
    }
}
