//
// CVU_Button.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// A CVU element for displaying a button
/// - Use the `onPress` property to provide a CVU Action for the button
struct CVU_Button: View {
    var nodeResolver: CVUUINodeResolver
    @EnvironmentObject var sceneController: SceneController
    @State var alertInfo: [String: Any] = [:]
    @State var alertAction: CVUAction? = nil
    @State var alertVisible: Bool = false
    @State var pendingActions: [CVUAction] = []
    
    func onPress() {
        guard let actions = self.nodeResolver.propertyResolver.actions(key: "onPress") else { return }
        self.pendingActions = actions
        executeNextPendingAction()
    }
    
    func openPopup(_ settings: [String:Any]) {
        guard let actions = settings["actions"] as? [CVUAction],
              let action = actions.first else {
            return
        }
    
        self.alertInfo = settings
        self.alertAction = action
        self.alertVisible = true
    }
    
    var body: some View {
        Button(action: onPress) {
            self.nodeResolver.childrenInForEach()
        }
        .buttonStyle(Style())
        .alert(isPresented: $alertVisible, content: {
            Alert(title: Text((alertInfo["title"] as? String) ?? "Alert"),
                  message: Text((alertInfo["text"] as? String) ?? "Message"),
                  primaryButton: .default(Text(alertAction?.getString("title", nodeResolver.context, sceneController.topMostContext) ?? ""), action: {
                    alertAction?.execute(sceneController: sceneController, context: nodeResolver.context, completion: { _ in
                        executeNextPendingAction()
                    })
                  }),
                  secondaryButton: .cancel())
        })
    }
    
    struct Style: ButtonStyle {
        func makeBody(configuration: Configuration) -> some View {
            configuration
                .label
                .shadow(radius: configuration.isPressed ? 4 : 0)
        }
    }
    
    private func executeNextPendingAction() {
        DispatchQueue.main.async {
            guard let action = pendingActions.first else {
                return
            }
            
            pendingActions.remove(at: 0)
            if let popUpAction = action as? CVUAction_OpenPopup {
                if let settings = popUpAction.setPopupSettings(sceneController: sceneController, context: nodeResolver.context) {
                    openPopup(settings)
                }
            } else {
                action.execute(sceneController: sceneController, context: nodeResolver.context) { _ in
                    executeNextPendingAction()
                }
            }
        }
    }
}
