//
// CVU_FileThumbnail.swift
// Copyright © 2020 memri. All rights reserved.

import Foundation
import SwiftUI

/// A CVU element for displaying a thumbnail for a file (eg. image, pdf)
struct CVU_FileThumbnail: View {
    var nodeResolver: CVUUINodeResolver

    var fileURL: URL? {
        guard let fileURI = nodeResolver.propertyResolver.fileUID(key: "file")
        else { return nil }
        return FileStorageController.getURLForFile(withUUID: fileURI)
    }

    var dimensions: CGSize {
        CGSize(width: max(30, nodeResolver.propertyResolver.cgFloat("width") ?? 100),
               height: max(30, nodeResolver.propertyResolver.cgFloat("width") ?? 100))
    }

    @ViewBuilder
    var body: some View {
        if let fileURL = fileURL {
            MemriFileThumbnailView(fileURL: fileURL, thumbnailDimensions: dimensions)
        }
        else {
            Image(systemName: "questionmark")
                .renderingMode(.template)
                .aspectRatio(contentMode: .fit)
                .foregroundColor(Color(.secondaryLabel))
                .frame(maxWidth: .infinity, maxHeight: .infinity)
                .layoutPriority(-1)
        }
    }
}
