//
// CVU_Text.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// A CVU element for displaying text
/// - Set the `text` property to the desired content
/// - Set the `font` property to change text appearance
/// - Set the `color` property to change text color
struct CVU_SubView: View {
    var nodeResolver: CVUUINodeResolver

    var content: String? {
        nodeResolver.propertyResolver.string("text")?.nilIfBlank
    }

    var title: String? {
        nodeResolver.propertyResolver.string("title")?.nilIfBlank
    }
    
    var viewContext: ViewContextController? {
        guard let viewDefinition = nodeResolver.propertyResolver.value("view")?.getSubdefinition(),
              case let .constant(.argument(rendererName)) = viewDefinition.properties["defaultRenderer"],
              case let .constant(.argument(viewName)) = viewDefinition.properties["viewName"],
              let datasource = viewDefinition.definitions.first(where: { $0.type == .datasource }) else {
            return nil
        }
        
        let initialItem: ItemRecord? = nodeResolver.propertyResolver.item("initialItem")
        let nodeItem = initialItem ?? nodeResolver.context.currentItem
             
        let cvuContext = CVUContext(currentItem: nodeItem, selector: nil, viewName: viewName, viewDefinition: viewDefinition)
        let queryConfig = DatabaseQueryConfig.queryConfigWith(context: cvuContext,
                                                              datasource: datasource)
        let config = ViewContext(viewName: viewName, rendererName: rendererName, viewDefinition: viewDefinition, query: queryConfig)
        let holder = ViewContextHolder(config)
        let viewControllerContext = ViewContextController(config: holder, databaseController: AppController.shared.databaseController, cvuController: AppController.shared.cvuController)

        return viewControllerContext
    }
    
    @ViewBuilder
    var renderer: some View {
        if let context = viewContext {
            SceneContentView(context: context).renderer
        } else {
            Text("No renderer selected").bold().frame(maxWidth: .infinity, maxHeight: .infinity)
        }
    }
    
    var body: some View {
        VStack(spacing: 0) {
            if let title = title {
                VStack(spacing: 10) {
                    Text(title)
                        .font(.system(size: 16, weight: .bold, design: .default))
                }
            }
            renderer
        }
    }
}
 
