//
// CVU_Text.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// A CVU element for displaying text
/// - Set the `text` property to the desired content
/// - Set the `font` property to change text appearance
/// - Set the `color` property to change text color
struct CVU_Text: View {
    var nodeResolver: CVUUINodeResolver

    var content: String? {
        nodeResolver.propertyResolver.string("text")?.nilIfBlank
    }

    var body: some View {
        content.map {
            Text($0)
                .fixedSize(horizontal: false, vertical: true)
        }
    }
}

/// A CVU element for displaying text with URLs and phone numbers auto-detected and clickable
/// - Set the `text` property to the desired content
/// - Set the `font` property to change text appearance
/// - Set the `color` property to change text color
struct CVU_SmartText: View {
    var nodeResolver: CVUUINodeResolver

    var content: String? {
        nodeResolver.propertyResolver.string("text")?.nilIfBlank
    }

    var body: some View {
        content
            .map {
                MemriSmartTextView(
                    string: $0,
                    font: nodeResolver.propertyResolver.font(),
                    color: nodeResolver.propertyResolver.color()
                )
            }
    }
}
