//
//  CVUUINodeResolver.swift
//  MemriDatabase
//
//  Created by T Brennan on 7/1/21.
//

import Foundation
import SwiftUI

/// This struct allows for a CVU node to be queried for the value of its properties.
struct CVUUINodeResolver {
    var context: CVUContext
    var lookup: CVULookupController
    var node: CVUUINode
    
    var db: DatabaseController

    func childrenInForEach() -> some View {
        return ForEach(node.children) { child in
            CVUElementView(nodeResolver: CVUUINodeResolver(context: context, lookup: lookup, node: child, db: db))
        }
    }
    
    func childrenInForEach(usingItem item: ItemRecord) -> some View {
        var newContext = context
        newContext.currentItem = item
        
        return ForEach(node.children) { child in
            CVUElementView(nodeResolver: CVUUINodeResolver(context: newContext, lookup: lookup, node: child, db: db))
        }
    }
    
    @ViewBuilder
    func firstChild() -> some View {
        if let child = node.children.first {
            CVUElementView(nodeResolver: CVUUINodeResolver(context: context, lookup: lookup, node: child, db: db))
        }
    }
    
    var propertyResolver: CVUPropertyResolver {
        CVUPropertyResolver(context: context, lookup: lookup, db: db, properties: node.properties)
    }
}

