//
// BottomBar.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// This view contains the contents of the 'Bottom bar' which is shown below renderer content.
struct BottomBarView: View {
    @ObservedObject var context: ViewContextController

    var onSearchPressed: () -> Void
    var onFilterButtonPressed: () -> Void

    var currentFilter: String? {
        context.searchString
    }

    @ViewBuilder
    var body: some View {
        VStack(spacing: 0) {
            Divider()
            HStack(spacing: 4) {
                HStack(spacing: 4) {
                    Button(action: onSearchPressed) {
                        HStack(spacing: 0) {
                            Image(systemName: "magnifyingglass")
                                .padding(.trailing, 7)
                            if let filter = currentFilter {
                                Text(filter).font(.caption)
                                    .foregroundColor(Color(.label))
                            }
                        }
                        .padding([.leading, .vertical], 10)
                        .contentShape(Rectangle())
                    }
                    if currentFilter != nil {
                        Button {
                            context.searchString = nil
                        } label: {
                            Image(systemName: "clear")
                                .foregroundColor(Color(.label))
                                .font(.caption)
                        }
                    }
                }
                Spacer()
                if let _ = context.focusedItem, let actions = context.itemPropertyResolver?.actions(key: "filterButtons"), actions.count > 0 {
                    ForEach(actions, id: \.id) { action in
                        ActionButton(action: action, context: context.getCVUContext(forItem: context.focusedItem))
                    }
                } else {
                    ForEach(context.viewDefinitionPropertyResolver.stringArray("filterButtons"),
                            id: \.id) { action in
                        if let action = cvuAction(named: action) {
                            ActionButton(action: action.init(vars: [:]), context: context.getCVUContext())
                        }
                    }
                }
            }
            .padding(.horizontal, 10)
            .font(Font.system(size: 20, weight: .medium))
            .background(Color(.secondarySystemBackground).edgesIgnoringSafeArea(.bottom))
        }
    }
}
