//
//  ContextualBottomBar.swift
//  Memri

import SwiftUI

struct ContextualBottomBar: View {
    @EnvironmentObject var sceneController: SceneController
    @ObservedObject var context: ViewContextController

    var shouldShow: Bool {
        sceneController.isInEditMode
    }

    var nonEmptySelection: Bool {
        return !context.selectedItems.isEmpty
    }

    var allItemsSelected: Bool {
        return context.allItemsSelected
    }
    
    @ViewBuilder
    var body: some View {
        if shouldShow {
            VStack(spacing: 0) {
                Divider()
                HStack {
                    if sceneController.isInEditMode {
                        if allItemsSelected {
                            ActionButton(action: CVUAction_DeselectAll.init(vars: [:]), context: context.getCVUContext())
                        }
                        else {
                            ActionButton(action: CVUAction_SelectAll.init(vars: [:]), context: context.getCVUContext())
                        }
                    }
                    Spacer()
                    if sceneController.isInEditMode {
                        ActionButton(action: CVUAction_Delete.init(vars: [:]), context: context.getCVUContext())
                    }
                }
                .padding(.horizontal)
                .padding(.vertical, 5)
            }
            .font(.body)
            .background(Color(.secondarySystemBackground))
        }
    }
}
