//
// SearchView.swift
// Copyright © 2020 memri. All rights reserved.

import Combine
import SwiftUI

/// This view is displayed when the user presses the search button
/// It moves itself to be above the keyboard automatically
struct SearchView: View {
    @ObservedObject var context: ViewContextController
    @Binding var isActive: Bool

    @ViewBuilder
    var body: some View {
        VStack(spacing: 0) {
            Divider()
            HStack {
                Image(systemName: "magnifyingglass")
                    .foregroundColor(Color(.systemFill))
                MemriTextField(
                    value: $context.searchString,
                    placeholder: "Search",
                    clearButtonMode: .always,
                    showPrevNextButtons: false,
                    isEditing: $isActive
                )
                .onEditingEnded {
                    DispatchQueue.main.async {
                        self.isActive = false
                    }
                }
            }
            .padding(.horizontal, 15)
            .padding(.vertical, 6)
        }
        .background(Color(.systemBackground))
        .modifier(KeyboardOffsetModifier())
        .background(Color(.systemBackground).edgesIgnoringSafeArea(.all))
        .transition(.opacity)
    }
}
