//
// BottomBar.swift
// Copyright © 2020 memri. All rights reserved.

import SwiftUI

/// This view provides the 'Navigation Bar' for the app interface
struct TopBarView: View {
    @ObservedObject
    var sceneController: SceneController
    var showEditButton: Bool {
        return (try? Settings.shared.getSetting("user/general/gui/showEditButton", type: Bool.self)) ?? true
    }

    @ViewBuilder
    var body: some View {
        VStack(spacing: 0) {
            HStack(spacing: 4) {
                HStack {
                    Button(action: {
                        withAnimation {
                            sceneController.navigationIsVisible.toggle()
                        }
                    }) {
                        Image(systemName: "line.horizontal.3")
                            .rotationEffect(sceneController.navigationIsVisible ? Angle(degrees: 90) : .zero)
                            .padding(10)
                            .contentShape(Rectangle())
                    }
                    if sceneController.canNavigateBack {
                        Button(action: {
                            sceneController.isInEditMode = false
                            sceneController.navigateBack()
                        }) {
                            Image(systemName: "chevron.backward.circle.fill")
                                .rotationEffect(sceneController.navigationIsVisible ? Angle(degrees: 90) : .zero)
                                .padding(10)
                                .contentShape(Rectangle())
                        }
                    }
                }
                .frame(minWidth: 100, alignment: .leading)
                if let context = sceneController.topMostContext,
                   let title = context.viewDefinitionPropertyResolver.string("title") ?? (context.focusedItem != nil ? context.itemPropertyResolver?.string("title") : "")  {
                    Text(title).font(.headline).frame(maxWidth: .infinity)
                } else {
                    Spacer()
                }
                HStack {
                    if let context = sceneController.topMostContext,
                       let editAction = sceneController.topMostContext?.viewDefinitionPropertyResolver.stringArray("editActionButton").first,
                       showEditButton
                    {
                        if let action = cvuAction(named: editAction) {
                            ActionButton(action: action.init(vars: ["icon": .constant(.string("pencil"))]), context: context.getCVUContext())
                        }
                    }

                    if let context = sceneController.topMostContext,
                       let actions = context.viewDefinitionPropertyResolver.actions(key: "actionButton")
                    {
                        ForEach(actions, id: \.id) { action in
                            ActionButton(action: action, context: context.getCVUContext())
                        }
                    }
                }
                    .frame(minWidth: 100, alignment: .trailing)
                }
                .padding(.horizontal, 10)
            .font(Font.system(size: 20, weight: .medium))
            .frame(minHeight: 48)
            .background(Color(.secondarySystemBackground).edgesIgnoringSafeArea(.top))
            
            Divider()
        }
    }
}
