//
//  ActionButton.swift
//  Memri
//
//  Created by Chaitanya Pandit on 24/03/21.
//

import Foundation
import SwiftUI

public struct ActionButton: View {
    @EnvironmentObject var sceneController: SceneController

    let action: CVUAction
    let context: CVUContext
    @State var icon: String? = nil
    @State var title: String? = nil

    public var body: some View {
        return Button(action: {
            self.action.execute(sceneController: sceneController, context: context, completion: nil)
            self.icon = action.getString("icon", context, sceneController.topMostContext)
            self.title = action.getString("title", context, sceneController.topMostContext)
        }) {
            if let icon = icon {
                Image(systemName: icon)
                .padding(10)
            }
            if let title = self.title {
                Text(title)
                    .padding(5)
            }
        }.onAppear() {
            self.icon = action.getString("icon", context, sceneController.topMostContext)
            self.title = action.getString("title", context, sceneController.topMostContext)
        }
    }
}

struct ActionPopupButton: View {
    var action: CVUAction_OpenViewByName

    @State var isShowing = false
    
    var body: some View {
        Button(action: {
            self.isShowing = true
        }) {
            Image(systemName: "plus")
            .padding(10)
        }
        .sheet(isPresented: $isShowing) {
            if let viewContext = action.getViewContext(context: CVUContext(viewName: action.viewName, rendererName: action.renderer)) {
                BrowserView(context: viewContext)
            } else {
                Text("TODO: ActionPopupButton")
            }
        }
    }
}
