//
//  MemriButton.swift
//  Memri
//
//  Created by Chaitanya Pandit on 19/03/21.
//

import Foundation
import SwiftUI

public struct MemriButton: View {

    var nodeResolver: CVUUINodeResolver
    let item: ItemRecord?
    var db: DatabaseController

    init(nodeResolver: CVUUINodeResolver, item: ItemRecord? = nil, db: DatabaseController) {
        self.nodeResolver = nodeResolver
        self.item = item
        self.db = db
    }

    public var body: some View {
        
        var title: String = ""
        var bgColor = Color(hex: "#ffffff")
        let foregroundColor = Color.white
        
        if let inputItem = item {
            switch inputItem.type {
            case "Person":
                if let firstName = inputItem.propertyValue("firstName")?.asString() {
                    title = firstName
                }
                if let lastName = inputItem.propertyValue("lastName")?.asString() {
                    title = "\(title) \(lastName)"
                }
                bgColor = Color(hex: "#3A5EB3")
            default:
                title = nodeResolver.propertyResolver.string("title") ?? ""
                break
            }
        }
   
        
        return Group {
            if item == nil {
                EmptyView()
            } else {
                HStack(spacing: 0) {
                    Text(nodeResolver.propertyResolver.string("title") ?? item?.type ?? "")
                        .padding(EdgeInsets(top: 3, leading: 8, bottom: 3, trailing: 8))
                        .background(Color(hex: "#afafaf"))
                        .foregroundColor(Color(hex: "#fff"))
                        .font(.system(size: 14, weight: .semibold))
                        .cornerRadius(20)

                    Text(title)
                        .padding(EdgeInsets(top: 3, leading: 5, bottom: 3, trailing: 9))
                        .foregroundColor(foregroundColor)
                        .font(.system(size: 14, weight: .semibold))
                        .zIndex(10)
                }
                .lineLimit(1)
                .background(bgColor)
                .cornerRadius(20)
                .compositingGroup()
            }
        }
    }
}
