//
// ChartHelper.swift
// Copyright © 2020 memri. All rights reserved.

import Foundation

/// A helper struct containing static functions for generating chart models from an array of items.
struct ChartHelper {
    static func generateXYChartSetFromItems(
        _ items: [ItemRecord],
        xAxis: (ItemRecord) -> Double?,
        yAxis: (ItemRecord) -> Double?,
        labelExpression: ((ItemRecord) -> String?)?
    ) -> ChartSetXY {
        let points = items.compactMap { (item) -> ChartPointXY? in
            guard
                let x: Double = xAxis(item),
                let y: Double = yAxis(item),
                !x.isNaN,
                !y.isNaN
            else { return nil }
            let label: String? = labelExpression.flatMap { $0(item) }
            return ChartPointXY(x: x, y: y, label: label, itemID: item.id.uid)
        }
        return ChartSetXY(points: points)
    }

    static func generateLabelledYChartSetFromItems(
        _ items: [ItemRecord],
        labelExpression: (ItemRecord) -> String?,
        yAxis: (ItemRecord) -> Double?,
        yAxisLabel: (ItemRecord) -> String?
    ) -> ChartSetLabelledY {
        let points = items.compactMap { (item) -> ChartPointLabelledY? in
            guard
                let label: String = labelExpression(item),
                let y: Double = yAxis(item),
                !y.isNaN
            else { return nil }
            return ChartPointLabelledY(label: label,
                                       y: y,
                                       yLabel: yAxisLabel(item),
                                       itemID: item.id.uid)
        }
        return ChartSetLabelledY(points: points)
    }
}
