//
// ChartModel.swift
// Copyright © 2020 memri. All rights reserved.

import Foundation

/// An enum representing supported chart types
enum ChartType: String, CaseIterable {
    case line
    case bar
    case pie
}

/// A set of data points for labelled-Y style charts (bar, pie)
struct ChartSetLabelledY {
    var points: [ChartPointLabelledY]
}

/// A set of data points for X-Y style charts (line)
struct ChartSetXY {
    var points: [ChartPointXY]
}

/// A single data point for labelled-Y style charts (bar, pie)
struct ChartPointLabelledY {
    var label: String
    var y: Double
    var yLabel: String?
    var itemID: String
}

/// A single data point for X-Y style charts (line)
struct ChartPointXY {
    var x: Double
    var y: Double
    var label: String?
    var itemID: String
}

/// Some additional information used by the chart views
struct ChartEntryInfo {
    var dataIndex: Int // Doesn't always equal index on chart (eg. line chart sorts by x)
    var label: String?
}
