//
// BarChartModel.swift
// Copyright © 2020 memri. All rights reserved.

import Charts
import Foundation
import SwiftUI

/// A model used for charts that have labelled values (eg. Bar, Pie)
struct ChartModelLabelledY {
    var sets: [ChartSetLabelledY]
    var hideGridLines: Bool = true
    var forceMinYOfZero: Bool = true
    var primaryColor = CVUColor.system(.systemBlue)
    var barLabelFont = UIFont.systemFont(ofSize: 13)
    var showValueLabels: Bool = true
    var valueLabelFont = UIFont.systemFont(ofSize: 14)

    func generateBarChartData() -> BarChartData {
        let dataSets: [BarChartDataSet] = sets.map { set in
            let dataSet = BarChartDataSet(entries: set.points.indexedElements().map { indexedPoint in
                BarChartDataEntry(
                    x: Double(indexedPoint.offset),
                    y: indexedPoint.y,
                    data: ChartEntryInfo(dataIndex: indexedPoint.index, label: indexedPoint.yLabel)
                )
            })
            dataSet.drawValuesEnabled = showValueLabels
            dataSet.valueFormatter = ChartLabelFormatter()
            dataSet.valueFont = valueLabelFont
            dataSet.setColor(primaryColor.uiColor)
            return dataSet
        }
        return BarChartData(dataSets: dataSets)
    }
    
    
    func generatePieChartData() -> PieChartData {
        let dataSets: [PieChartDataSet] = sets.map { set in
            let dataSet = PieChartDataSet(entries: set.points.indexedElements().map { indexedPoint in
                PieChartDataEntry(
                    value: indexedPoint.y,
                    label: indexedPoint.label,
                    data: ChartEntryInfo(dataIndex: indexedPoint.index, label: indexedPoint.yLabel)
                )
            })
            dataSet.drawValuesEnabled = showValueLabels
            dataSet.valueFormatter = ChartLabelFormatter()
            dataSet.valueFont = valueLabelFont
            dataSet.setColors(ChartColorTemplates.pastel(), alpha: 1)
            return dataSet
        }
        return PieChartData(dataSets: dataSets)
    }

    func getLabels() -> [String] {
        sets.first?.points.map(\.label) ?? []
    }
}
