//
// ImagePickerView.swift
// Copyright © 2020 memri. All rights reserved.

import PhotosUI
import SwiftUI

struct ImagePickerView: UIViewControllerRepresentable {
    @Environment(\.presentationMode) var presentationMode
    var onCompletion: (UIImage?) -> Void

    func makeUIViewController(context: Context) -> PHPickerViewController {
        var configuration = PHPickerConfiguration()
        configuration.filter = .images
        configuration.selectionLimit = 1

        let picker = PHPickerViewController(configuration: configuration)
        picker.delegate = context.coordinator
        return picker
    }

    func updateUIViewController(_ uiViewController: PHPickerViewController, context: Context) {}

    func makeCoordinator() -> Coordinator {
        Coordinator(self)
    }

    class Coordinator: NSObject, UINavigationControllerDelegate, PHPickerViewControllerDelegate {
        let parent: ImagePickerView

        init(_ parent: ImagePickerView) {
            self.parent = parent
        }

        func picker(_ picker: PHPickerViewController, didFinishPicking results: [PHPickerResult]) {
            let onCompletion: (UIImage?) -> Void = { image in
                DispatchQueue.main.async {
                    self.parent.onCompletion(image)
                    self.parent.presentationMode.wrappedValue.dismiss()
                }
            }
            guard let itemProvider = results.first?.itemProvider,
                  itemProvider.canLoadObject(ofClass: UIImage.self)
            else {
                onCompletion(nil)
                return
            }
            itemProvider.loadObject(ofClass: UIImage.self) { image, _ in
                let image = image as? UIImage
                onCompletion(image)
            }
        }
    }
}
