//
//  ChartRendererSettingsView.swift
//  Memri
//
//  Created by T Brennan on 3/2/21.
//

import Foundation
import SwiftUI

struct ChartRendererSettingsView: View {
    @ObservedObject var context: ViewContextController

    var colorBinding: Binding<Color> {
        Binding<Color>(get: {
            context.rendererDefinitionPropertyResolver.color(key: "color")?.color ?? .blue
        }, set: { newValue in
            let hex = UIColor(newValue).hexString()
            context.setRendererProperty(renderer: "chart", property: "color", value: .constant(.colorHex(hex)))
        })
    }
    
    var titleBinding: Binding<String?> {
        Binding<String?>(get: {
            context.rendererDefinitionPropertyResolver.string("title")
        }, set: { newValue in
            context.setRendererProperty(renderer: "chart", property: "title",
                                        value: .constant(newValue.map { .string($0) } ?? .nil))
        })
    }
    
    var subtitleBinding: Binding<String?> {
        Binding<String?>(get: {
            context.rendererDefinitionPropertyResolver.string("subtitle")
        }, set: { newValue in
            context.setRendererProperty(renderer: "chart", property: "subtitle",
                                        value: .constant(newValue.map { .string($0) } ?? .nil))
        })
    }
    
    var body: some View {
        ColorPicker("Primary color", selection: colorBinding)
        HStack {
            Text("Title:")
            MemriTextField(value: titleBinding)
                .padding(4)
                .background(RoundedRectangle(cornerRadius: 4).fill(Color(.secondarySystemBackground)))
        }
        HStack {
            Text("Subtitle:")
            MemriTextField(value: subtitleBinding)
                .padding(4)
                .background(RoundedRectangle(cornerRadius: 4).fill(Color(.secondarySystemBackground)))
        }
    }
}
