//
//  CalendarRendererModel.swift
//  Memri
//
//  Created by T Brennan on 30/1/21.
//

import Foundation

/// This struct is used to create and hold a pre-processed representation of data for the calendar view (eg. grouped by day)
struct CalendarCalculations {
    init(
        calendarHelper: CalendarHelper,
        data: [ItemRecord],
        dateResolver: (ItemRecord) -> Date?
    ) {
        let datesWithItems: [Date: [ItemRecord]] = data.reduce(into: [:]) { result, item in
            guard let dateTime = dateResolver(item)
            else { return }
            let date = calendarHelper.startOfDay(for: dateTime)
            result[date] = (result[date] ?? []) + [item]
        }
        self.datesWithItems = datesWithItems
        start = datesWithItems.keys.min()
            .flatMap { CalendarHelper().startOfMonth(for: $0) } ?? Date()
        end = datesWithItems.keys.max().flatMap { CalendarHelper().endOfMonth(for: $0) } ?? Date()
    }

    var datesWithItems: [Date: [ItemRecord]]
    var start: Date
    var end: Date

    func hasItemOnDay(_ day: Date) -> Bool {
        datesWithItems.keys.contains(day)
    }

    func itemsOnDay(_ day: Date) -> [ItemRecord] {
        datesWithItems[day] ?? []
    }
}
