//
// FileRenderer.swift
// Copyright © 2020 memri. All rights reserved.

import Foundation
import SwiftUI

/// The file viewer renderer
/// This presents the data items in a file viewer, that can page horizontally between files
/// The CVU for ItemType > FileViewer {...} must define a `file` expression pointing to a `File` item
/// Optionally the CVU for ItemType > FileViewer {...} can define an `itemTitle` expression
struct FileRendererView: View {
    @ObservedObject var viewContext: ViewContextController

    var files: [FileViewerItem] {
        viewContext.items.compactMap { item -> FileViewerItem? in
            guard let resolver = viewContext.nodePropertyResolver(for: item),
                  let fileURL = resolver.fileURL(key: "file")
            else { return nil }
            return FileViewerItem(url: fileURL,
                                  title: resolver.string("itemTitle"))
        }
    }
    
    var isFullScreen: Binding<Bool> {
        Binding<Bool>(
            get: { [weak viewContext] in
                viewContext?.isFullScreen ?? false
            }, set: { [weak viewContext] in
                viewContext?.isFullScreen = $0
            }
        )
    }
    
    var body: some View {
        MemriFileViewController(
            files: files,
            initialIndex: viewContext.focusedIndex,
            navBarHiddenBinding: isFullScreen
        )
    }
}
