import SwiftUI
import ASCollectionView

struct GridRendererLayout_Grid: GridRendererLayout {
    init(renderConfigResolver: CVUPropertyResolver) {
        self.resolver = renderConfigResolver
    }
    
    var resolver: CVUPropertyResolver
    
    let layoutName: String = "grid"
    var layoutStateID: String { "grid\(scrollDirection.rawValue)" }
    
    var minColumnSize: CGFloat {
        resolver.cgFloat("columnSize") ?? 100
    }
    
    var columns: Int? {
        resolver.int("columns")
    }
    
    var spacing: CGPoint {
        resolver.spacing ?? CGPoint(x: 10, y: 10)
    }
    
    var scrollDirection: UICollectionView.ScrollDirection {
        switch resolver.string("scrollDirection") {
        case "horizontal": return .horizontal
        default: return .vertical
        }
    }
    
    var layout: ASCollectionLayout<Int> {
        ASCollectionLayout(scrollDirection: scrollDirection, interSectionSpacing: 0) {
            ASCollectionLayoutSection { environment in
                let contentInsets = resolver.nsEdgeInset  ?? .init(top: 10, leading: 10, bottom: 10, trailing: 10)
                let numberOfColumns: Int

                switch scrollDirection {
                case .horizontal:
                    numberOfColumns = columns ?? max(1, Int((environment.container.effectiveContentSize.height + spacing.y - contentInsets.top - contentInsets.bottom) / (minColumnSize + spacing.y)))
                    
                    let ySpacing = spacing.y
                    let estimatedGridBlockSize = (environment.container.effectiveContentSize
                        .height - contentInsets.top - contentInsets
                        .bottom - ySpacing *
                        (CGFloat(numberOfColumns) - 1)) /
                        CGFloat(numberOfColumns)

                    let item = NSCollectionLayoutItem(
                        layoutSize: NSCollectionLayoutSize(
                            widthDimension: .estimated(estimatedGridBlockSize),
                            heightDimension: .fractionalHeight(1.0)
                        )
                    )

                    let itemsGroup = NSCollectionLayoutGroup.vertical(
                        layoutSize: NSCollectionLayoutSize(
                            widthDimension: .estimated(estimatedGridBlockSize),
                            heightDimension: .fractionalHeight(1.0)
                        ),
                        subitem: item, count: numberOfColumns
                    )
                    itemsGroup.interItemSpacing = .fixed(ySpacing)

                    let section = NSCollectionLayoutSection(group: itemsGroup)
                    section.interGroupSpacing = spacing.x
                    section.contentInsets = contentInsets
                    return section
                default:
                    numberOfColumns = columns ?? max(1, Int((environment.container.effectiveContentSize.width + spacing.x - contentInsets.leading - contentInsets.trailing) / (minColumnSize + spacing.x)))
                    
                    let xSpacing = spacing.x
                    let estimatedGridBlockSize = (environment.container.effectiveContentSize
                        .width - contentInsets.leading - contentInsets
                        .trailing - xSpacing *
                        (CGFloat(numberOfColumns) - 1)) /
                        CGFloat(numberOfColumns)

                    let item = NSCollectionLayoutItem(
                        layoutSize: NSCollectionLayoutSize(
                            widthDimension: .fractionalWidth(1.0),
                            heightDimension: .estimated(estimatedGridBlockSize)
                        )
                    )

                    let itemsGroup = NSCollectionLayoutGroup.horizontal(
                        layoutSize: NSCollectionLayoutSize(
                            widthDimension: .fractionalWidth(1.0),
                            heightDimension: .estimated(estimatedGridBlockSize)
                        ),
                        subitem: item, count: numberOfColumns
                    )
                    itemsGroup.interItemSpacing = .fixed(xSpacing)

                    let section = NSCollectionLayoutSection(group: itemsGroup)
                    section.interGroupSpacing = spacing.y
                    section.contentInsets = contentInsets
                    return section
                }
            }
        }
    }
}
