import SwiftUI
import ASCollectionView

struct GridRendererLayout_Waterfall: GridRendererLayout {
    init(renderConfigResolver: CVUPropertyResolver) {
        self.resolver = renderConfigResolver
    }
    
    var resolver: CVUPropertyResolver
    
    let layoutName = "waterfall"
    
    /// A value that changes when the layout needs to be reloaded
    var layoutStateID: String { layoutName }
    
    var minColumnSize: CGFloat {
        resolver.cgFloat("columnSize") ?? 100
    }
    
    var spacing: CGPoint {
        resolver.spacing ?? CGPoint(x: 10, y: 10)
    }
    
    var scrollDirection: UICollectionView.ScrollDirection {
        switch resolver.string("scrollDirection") {
        case "horizontal": return .horizontal
        default: return .vertical
        }
    }
    
    var layout: ASCollectionLayout<Int> {
        ASCollectionLayout(createCustomLayout: ASWaterfallLayout.init) { layout in
            layout.columnSpacing = spacing.x
            layout.itemSpacing = spacing.y
            layout
                .numberOfColumns =
                .adaptive(minWidth: minColumnSize)
        }
    }
}
class GridRendererLayout_Waterfall_Delegate: ASCollectionViewDelegate, ASWaterfallLayoutDelegate {
    func heightForHeader(sectionIndex _: Int) -> CGFloat? {
        0
    }

    let heights: [CGFloat] = [1.5, 1.0, 0.75, 1.75, 0.6]
    /// We explicitely provide a height here. If providing no delegate, this layout will use auto-sizing, however this causes problems if rotating the device (due to limitaitons in UICollecitonView and autosizing cells that are not visible)
    func heightForCell(
        at indexPath: IndexPath,
        context: ASWaterfallLayout.CellLayoutContext
    ) -> CGFloat {
        //        guard let item: Item = getDataForItem(at: indexPath) else { return 100 }
        let rand = indexPath.item % heights.count
        return context.width * (heights[safe: rand] ?? 1)
    }
 }
