//
//  ContentView.swift
//  memri
//
//  Created by T Brennan on 7/12/20.
//  Copyright © 2020 memri. All rights reserved.
//

import Foundation
import SwiftUI

/// This is the view used to display the browser content of each scene
struct SceneView: View {
    @ObservedObject var sceneController: SceneController
    @GestureState var filterPanelGestureOffset: CGFloat = .zero
    @GestureState var contextPaneGestureOffset: CGFloat = .zero

    var body: some View {
        NavigationWrapperView(sceneController: sceneController) {
            ZStack(alignment: .bottom) {
                VStack(spacing: 0) {
                    TopBarView(sceneController: sceneController)
                    NavigationHolder(controller: sceneController.navigationController)
                        .edgesIgnoringSafeArea(.bottom)
                }
                if sceneController.filterPanelIsVisible,
                   let currentContext = sceneController.topMostContext {
                    Color.black.opacity(0.15)
                        .onTapGesture {
                            withAnimation {
                                sceneController.filterPanelIsVisible = false
                            }
                        }
                        .gesture(DragGesture()
                            .updating($filterPanelGestureOffset) { value, state, _ in
                                state = max(0, value.translation.height)
                            }.onEnded { value in
                                if value.predictedEndTranslation.height > 20 {
                                    withAnimation {
                                        sceneController.filterPanelIsVisible = false
                                    }
                                }
                            })
                    VStack {
                        Capsule()
                            .fill(Color(.secondarySystemBackground))
                            .frame(width: 40, height: 5)
                            .frame(maxWidth: .infinity)
                            .frame(height: 15)
                        FilterPanelView(context: currentContext)
                    }
                    .offset(y: filterPanelGestureOffset)
                    .transition(.move(edge: .bottom))
                    .zIndex(9)
                }
                if sceneController.contextPaneVisible,
                   let currentContext = sceneController.topMostContext {
                    ContextPane(context: currentContext, onPaneClosed: {
                        sceneController.contextPaneVisible = false
                    })
                    .zIndex(9)
                }

            }
        }
    }
}

