//
//  SettingsPane.swift
//  Memri
//

import Foundation
import SwiftUI

struct SettingsPane: View {
    @Environment(\.presentationMode) var presentationMode: Binding<PresentationMode>

    @State var isLocationEnabled: Bool = AppController.shared.permissionController.locationPermission
    @State var isPhotosAvailable: Bool = AppController.shared.permissionController.libraryPermission
    @State var isCameraAvailable: Bool = AppController.shared.permissionController.cameraPermission
    @State var isMicrophoneAvailable: Bool = AppController.shared.permissionController.microPhonePermission
    @State var isContactsAvailable: Bool = AppController.shared.permissionController.contactsPermission
    
    // .keyboardType(.numberPad)
    var body: some View {
        NavigationView {
            Form {
                NavigationLink(destination: Form {
                    Section(
                        header: Text("Pod Connection"),
                        footer: Text("Never give out these details to anyone")
                            .font(.system(size: 11, weight: .regular))
                    ) {
                        HStack {
                            Text("Host:")
                                .frame(width: 100, alignment: .leading)
                            MemriTextField(value: Settings.shared.getBinding("user/pod/host") as Binding<String>)
                        }
                        HStack {
                            Button(action: {
                                try? AppController.shared.syncController.sync()
                            }) {
                                Text("Connect")
                            }
                        }
                    }
                    Section(
                        header: Text("Syncing")
                    ) {
                        Toggle(isOn: Settings.shared.getBinding("device/upload/cellular")) {
                            Text("Enable upload of images while on cellular")
                        }
                    }
                }) {
                    Text("Pod Connection")
                }

                NavigationLink(destination: Form {
                    Section(
                        header: Text("User Interface"),
                        footer: Text("Show 'xx time ago' in place of dates less than 36 hours ago")
                            .font(.system(size: 11, weight: .regular))
                    ) {
                        Toggle(isOn: Settings.shared.getBinding("user/general/gui/showEditButton")) {
                            Text("Always show edit button")
                        }
                        Toggle(isOn: Settings.shared.getBinding("user/general/gui/showDateAgo")) {
                            Text("Enable time ago")
                        }
                    }
                }) {
                    Text("User Interface")
                }

                NavigationLink(destination: Form {
                    Section(
                        header: Text("Sensors")
                    ) {
                        Toggle(isOn: Settings.shared.getBinding("device/sensors/location/track")) {
                            Text("Track and store location")
                        }
                    }
                }) {
                    Text("Sensors")
                }

                NavigationLink(destination: Form {
                    Section(header: Text("Internationalization")) {
                        MemriTextField(
                            value: Settings.shared.getBinding("user/formatting/date") as Binding<String>
                        )
                    }
                }) {
                    Text("Internationalization")
                }

                NavigationLink(destination: Form {
                    Section(
                        header: Text("Debug Settings")
                    ) {
                        Toggle(isOn: Settings.shared.getBinding("device/debug/autoShowErrorConsole")) {
                            Text("Automatically pop up the debug console on errors")
                        }
                        Toggle(isOn: Settings.shared.getBinding("device/debug/autoReloadCVU")) {
                            Text("Automatically reload CVU when it changes")
                        }
                    }

                    Section(
                        header: Text("Debug Actions")
                    ) {
                        HStack {
                            Button(action: {
                                try? AppController.shared.syncController.sync()
                            }) {
                                Text("Sync To Pod")
                            }
                        }
                    }
                }) {
                    Text("Debug")
                }

                NavigationLink(destination: Form {
                    Section(
                        header: Text("Permissions")
                    ) {
                        Toggle(isOn: $isLocationEnabled) {
                            Text("Location Access")
                        }
                        .onChange(of: isLocationEnabled) { value in
                            if value {
                                AppController.shared.permissionController.requestLocation { status in
                                    isLocationEnabled = status
                                }
                            }
                        }

                        Toggle(isOn: $isPhotosAvailable) {
                            Text("Photos Access")
                        }
                        .onChange(of: isPhotosAvailable) { value in
                            if value {
                                AppController.shared.permissionController.requestPhotos { status in
                                    isPhotosAvailable = status
                                }
                            }
                        }

                        Toggle(isOn: $isCameraAvailable) {
                            Text("Camera Access")
                        }
                        .onChange(of: isCameraAvailable) { value in
                            if value {
                                AppController.shared.permissionController.requestCamera { status in
                                    isCameraAvailable = status
                                }
                            }
                        }

                        Toggle(isOn: $isMicrophoneAvailable) {
                            Text("Microphone Access")
                        }
                        .onChange(of: isMicrophoneAvailable) { value in
                            if value {
                                AppController.shared.permissionController.requestMicrophone { status in
                                    isMicrophoneAvailable = status
                                }
                            }
                        }
                        
                        Toggle(isOn: $isContactsAvailable) {
                            Text("Contacts Access")
                        }
                        .onChange(of: isContactsAvailable) { value in
                            if value {
                                AppController.shared.permissionController.requestContacts { status in
                                    isContactsAvailable = status
                                }
                            }
                        }
                    }
                }) {
                    Text("Permissions")
                }

            }
            .navigationBarItems(leading:
                Button(action: { self.presentationMode.wrappedValue.dismiss() }) {
                    Text("Close")
                })
            .navigationBarTitle(Text("Settings"), displayMode: .inline)
        }
    }
}

