//
//  SetupScreenModel.swift
//  Memri
//
//  Created by T Brennan on 28/1/21.
//

import Foundation

/// A model struct used by the setup screen to hold configuration until the user completes the setup
struct SetupScreen_Model {
    let defaultPodURL = "http://0.0.0.0:3030"
    var podURL: String?
    var podPrivateKey: String?
    var podPublicKey: String?
    var podDatabaseKey: String?
    var setupAsNewPod: Bool = true
    var useDemoData: Bool = true

    var state: PodSetupState = .idle
    
    func getPodURL() -> String {
        podURL?.nilIfBlank ?? defaultPodURL
    }
    
    var isValidToProceedToConnect: Bool {
        if setupAsNewPod {
            return true
        }
        
        guard podPrivateKey?.nilIfBlank != nil,
              podPublicKey?.nilIfBlank != nil,
              podDatabaseKey?.nilIfBlank != nil else { return false }
        return true
    }
    
    enum PodSetupState: Equatable {
        case idle
        case loading
        case error(String)
    }
    
    func getSetupConfig(localOnly: Bool) -> SetupConfig? {
        if localOnly {
            return .local
        } else if setupAsNewPod {
            let config = SetupConfig.NewPodConfig(podURL: podURL ?? defaultPodURL)
            return .newPod(config)
        } else {
            guard let privateKey = podPrivateKey?.nilIfBlank, let publicKey = podPublicKey?.nilIfBlank, let databaseKey = podDatabaseKey?.nilIfBlank
            else { return nil }
            let config = SetupConfig.ExistingPodConfig(podURL: podURL ?? defaultPodURL, podPrivateKey: privateKey, podPublicKey: publicKey, podDatabaseKey: databaseKey)
            return .existingPod(config)
        }
    }
}
