//
//  ExprLookupTests.swift
//  MemriDatabaseTests
//
//  Created by T Brennan on 24/12/20.
//

@testable import Memri
import GRDB
import XCTest

class ExprLookupTests: XCTestCase {

    override func setUpWithError() throws {
        // Put setup code here. This method is called before the invocation of each test method in the class.
    }

    override func tearDownWithError() throws {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
    }

    private func parse(_ snippet: String, stringMode: Bool = false) throws -> CVUValue.ExpressionNode {
        let lexer = CVUExpressionLexer(input: snippet, startInStringMode: stringMode)
        let tokens = try lexer.tokenize()
        let parser = CVUExpressionParser(tokens)
        return try parser.parse()
    }
    
    let databaseController: DatabaseController = {
        let db = DatabaseController()
        try! DemoData.importDemoData(databaseController: db, throwIfAgainstSchema: false)
        return db
    }()
    
    lazy var lookupController: CVULookupController = {
        return CVULookupController()
    }()
    
    private func interpretAsBool(_ expr: CVUValue.ExpressionNode, context: CVUContext = .init()) throws -> Bool? {
        lookupController.resolve(value: CVUValue.expression(expr), context: context, db: databaseController)
    }
    
    private func interpretAsDouble(_ expr: CVUValue.ExpressionNode, context: CVUContext = .init()) throws -> Double? {
        lookupController.resolve(value: CVUValue.expression(expr), context: context, db: databaseController)
    }
    
    private func interpretAsString(_ expr: CVUValue.ExpressionNode, context: CVUContext = .init()) throws -> String? {
        lookupController.resolve(value: CVUValue.expression(expr), context: context, db: databaseController)
    }
    
    func testStringModeStartWithString() throws {
        let snippet = "Note title: {.title}"
        
        let expr = try parse(snippet, stringMode: true)
        let note = try databaseController.read { (db) in
            try ItemRecord.filter(ItemRecord.Columns.type == "Note").fetchOne(db)
        }
        let noteString = note!.propertyValue("title", db: databaseController)!.asString()
        let context = CVUContext(currentItem: note!)
        let result = try interpretAsString(expr, context: context)

        XCTAssertEqual(result, "Note title: \(noteString!)")
    }
    
    
    // TODO: fix tests
    /*

    func testStringModeStartWithStringTwo() throws {
        let snippet = "Filename: {.file.filename}"
        
        let expr = try parse(snippet, stringMode: true)
        let note = try databaseController.read { (db) in
            try ItemRecord.filter(ItemRecord.Columns.type == "Note").fetchAll(db)[1]
        }
        let context = CVUContext(currentItem: note)
        let result = try interpretAsString(expr, context: context)

        XCTAssertEqual(result, "Filename: unsplash-image2")
    }
    
    func testStringModeStartWithStringArray() throws {
        let snippet = "Filenames: {.file[].filename.joined()}"
        
        let expr = try parse(snippet, stringMode: true)
        let note = try databaseController.read { (db) in
            try ItemRecord.filter(ItemRecord.Columns.type == "Note").fetchAll(db)[1]
        }
        let context = CVUContext(currentItem: note)
        let result = try interpretAsString(expr, context: context)

        XCTAssertEqual(result, "Filenames: unsplash-image2, karen-penroz-06ZTGDcAQFs-unsplash, and szabo-viktor-tarTTEaiIx4-unsplash")
    }
     
    func testStringModeStartWithStringArrayTwo() throws {
        let snippet = "Filenames: {.~includes[].file.filename.joined(\" !! \")}"
        
        let expr = try parse(snippet, stringMode: true)
        let person = try databaseController.read { (db) in
            try ItemRecord.filter(ItemRecord.Columns.type == "Person").fetchOne(db)
        }
        let context = CVUContext(currentItem: person!)
        let result = try interpretAsString(expr, context: context)

        XCTAssertEqual(result, "Filenames: fake-person10 !! unsplash-image17 !! unsplash-image18 !! fake-person58 !! unsplash-image19 !! fake-person56 !! unsplash-image29 !! unsplash-image55 !! fake-person52 !! unsplash-image19")
    }
    
    func testStringModeSubexp() throws {
        let snippet = #"Filename: {.~includes[].file[.filename = "unsplash-image17"].filename}"#
        
        let expr = try parse(snippet, stringMode: true)
        let person = try databaseController.read { (db) in
            try ItemRecord.filter(ItemRecord.Columns.type == "Person").fetchOne(db)
        }
        let context = CVUContext(currentItem: person!)
        let result = try interpretAsString(expr, context: context)

        XCTAssertEqual(result, "Filename: unsplash-image17")
    }
 */
}
