//
//  DatabaseControllerTests.swift
//  MemriDatabaseTests
//
//  Created by T Brennan on 16/12/20.
//

import XCTest
@testable import Memri

class SchemaControllerTests: XCTestCase {
    
    override func setUpWithError() throws {
        // Put setup code here. This method is called before the invocation of each test method in the class.
    }

    override func tearDownWithError() throws {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
    }

    // MARK: Schema tests
    
    func testDecodeSchemaProperty() throws {
        let propertyString = #"{ "item_type": "Review", "property": "title", "value_type": "string" }"#
        let property = try JSONDecoder().decode(SchemaProperty.self, from: propertyString.data(using: .utf8)!)
        XCTAssertEqual(property.itemType, "Review")
        XCTAssertEqual(property.property, "title")
        XCTAssertEqual(property.valueType, .string)
    }
    
    func testEncodeSchemaProperty() throws {
        let propertyString = #"{ "item_type": "Review", "property": "title", "value_type": "string" }"#
        let property = try JSONDecoder().decode(SchemaProperty.self, from: propertyString.data(using: .utf8)!)
        let encoded = String(data: try JSONEncoder().encode(property), encoding: .utf8)
        XCTAssertEqual(encoded,  #"{"value_type":"string","property":"title","item_type":"Review"}"#)
    }
    
    func testDecodeSchemaEdge() throws {
        let edgeString = #"{ "source_type": "Photo", "edge": "file", "target_type": "File" }"#
        let edge = try JSONDecoder().decode(SchemaEdge.self, from: edgeString.data(using: .utf8)!)
        XCTAssertEqual(edge.sourceType, "Photo")
        XCTAssertEqual(edge.edge, "file")
        XCTAssertEqual(edge.targetType, "File")
        XCTAssertEqual(edge.allowsMultiple, false)
    }
    
    func testDecodeSchemaEdgeArray() throws {
        let edgeString = #"{ "source_type": "Email", "edge": "attachment", "target_type": "File[]" }"#
        let edge = try JSONDecoder().decode(SchemaEdge.self, from: edgeString.data(using: .utf8)!)
        XCTAssertEqual(edge.sourceType, "Email")
        XCTAssertEqual(edge.edge, "attachment")
        XCTAssertEqual(edge.targetType, "File")
        XCTAssertEqual(edge.allowsMultiple, true)
    }
    
    func testEncodeSchemaEdge() throws {
        let edgeString = #"{ "source_type": "Photo", "edge": "file", "target_type": "File" }"#
        let edge = try JSONDecoder().decode(SchemaEdge.self, from: edgeString.data(using: .utf8)!)
        let encoded = String(data: try JSONEncoder().encode(edge), encoding: .utf8)
        XCTAssertEqual(encoded,  #"{"edge":"file","source_type":"Photo","target_type":"File"}"#)
    }
    
    func testEncodeSchemaEdgeArray() throws {
        let edgeString = #"{ "source_type": "Email", "edge": "attachment", "target_type": "File[]" }"#
        let edge = try JSONDecoder().decode(SchemaEdge.self, from: edgeString.data(using: .utf8)!)
        let encoded = String(data: try JSONEncoder().encode(edge), encoding: .utf8)
        XCTAssertEqual(encoded, #"{"edge":"attachment","source_type":"Email","target_type":"File[]"}"#)
    }
    
    func testLoadSchemaFromJSON() throws {
        _ = try Schema.loadFromFile()
    }
    
    func testLoadSchemaFromJSONPerformance() {
        self.measure {
            _ = try! Schema.loadFromFile()
        }
    }
}
